/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G139
* Tool-Chain   : MDK(armcc)
* Description  : This file is a sample to calculate Pi value by Monte Carlo method with TRNG module.
* Creation Date: 2022/10/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
#pragma clang diagnostic ignored "-Wunused-parameter"
#pragma clang diagnostic ignored "-Wunused-variable"
#pragma clang diagnostic ignored "-Wdouble-promotion"

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <math.h>
#include <stdio.h>
#include <stdint.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "clk.h"
#include "sci.h"
#include "trng.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
static volatile float pi=0.0;

void SysTick_Handler(void);
void delayMS(uint32_t n);

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
 {
	uint32_t msCnt; 	// count value of 1ms
	MD_STATUS status;
    uint32_t begin_sys,end_sys;
    uint32_t i;
    int32_t val;
    float x,y;
    float r=1.0;
    uint32_t over_1,less_1;

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 


//-----------------------------------------------------------------------
// fIH=16MHz (Option Byte(402H) = 0xEA)
//-----------------------------------------------------------------------
    //fPLL=128MHz	
    CLK_PLL_Setting(PLL_SR_fIH,PLL_DIV_2,PLL_MUL_16);
    CLK_PLL_Start();
    CLK_Fclk_Select(MAINCLK_FPLL);
    while((CGC->MCKC & 0x80) == 0x00);	        //wait for CKSTR = 1

//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}
	printf("This sample calculates Pi value by Monte Carlo method with FPU and TRNG module.\n");

//-----------------------------------------------------------------------
// Init TRNG 
//----------------------------------------------------------------------- 
    TRNG_Init(TRNGCKS_CKS_DIV_32);      // TRNG need operation clock under 4MHz

    val=0x00000000;
    over_1=0;
    less_1=0;
    for(i=0;i<=10000;i++){
        // 1) get a random x-coordinate:
        val=(int32_t)TRNG_GetRandomNumber();
        //printf("[%04d] Get random number = %d \n",i*2,val);
        x=(float)val/2147483648;

        // 2) get a random y-coordinate:
        val=(int32_t)TRNG_GetRandomNumber();
        //printf("[%04d] Get random number = %d \n",i*2+1,val);
        y=(float)val/2147483648;

        //printf("[%04d] (x=%3.8f,y=%3.8f) \n",i,x,y);

        // 3) Calculate this point whether is in the circle with r=1 or not?
        #if 0
            begin_sys = SysTick->VAL;
            r=sqrt(x*x+y*y);
            end_sys = SysTick->VAL;
            printf("Calculate [r=sqrt(x*x+y*y]=%3.8f; use systick = %d\n",r,(begin_sys-end_sys));
        #else
            //r=x*x+y*y;            // ?r^2 > 1.0
            r=sqrt(x*x+y*y);        // ?r   > 1.0
        #endif
        if(r>1.0){
            over_1++;
            //printf("r=%3.8f > 1.0\n",r);
        }
        else{
            less_1++;
            //printf("r=%3.8f < 1.0\n",r);
        }

        // 4) output pi value
        if(i%100==0){
            pi=less_1*4.0/i;    // calculate pi value
            printf("[%d time calculate] pi=%3.8f\n",i,pi);
        }
    }
	while(1) 
	{
	}
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

