/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G439
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
#pragma clang diagnostic ignored "-Wunused-parameter"
#pragma clang diagnostic ignored "-Wunused-variable"

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "adc.h"
#include "elc.h"
#include "sci.h"
#include "intp.h"


/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);

static double voltage=5.0;
static uint16_t get_value[8];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}


int main()
{
	uint32_t msCnt; 	// count value of 1ms
	MD_STATUS status;
	uint32_t i;
    uint16_t * pBuf;
	
//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 

//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);
	}
	printf("Hello, UART\n");
#endif

    PORT->PMC  &= ~0x4807;  // PC14,PC11&PC02-P00 output mode
    PORT->PCLRC =  0x4807;

    // Configure ADC2
	adc_initial(ADC2,0);

    adc_operation_mode(ADC2, AD_GROUP_SCAN_MODE);           // group scan mode
    adc_gp_scan_ch_setting(ADC2, AD_GROUP_A, ADCx_CH0+ADCx_CH1);    // group a chennal setting (PA06/ANI200,PA07/ANI201)
    adc_gp_scan_ch_setting(ADC2, AD_GROUP_B, ADCx_CH6+ADCx_CH7);    // group b chennal setting
    adc_gp_scan_ch_setting(ADC2, AD_GROUP_C, ADCx_CH8+ADCx_CH9);    // group c chennal setting

    adc_syn_trigger_setting(ADC2, AD_GROUP_A, TRGA0N);              // TRGA0N: ELSR04
    adc_syn_trigger_setting(ADC2, AD_GROUP_B, TRGA1N);              // TRGA1N: ELSR05
    adc_syn_trigger_setting(ADC2, AD_GROUP_C, TRGA2N);              // TRGA2N: ELSR06

    // Configure ELSR04-06 of ELC function. (INTP0-INTP2 are used to trigger ADC group scan, here.)
    ELC_Start(ELC_EVENT04_LINK_ADCEVENT0, ELC_EVENT_INTP0_IRQn);    // ELSR04: INTP0
    ELC_Start(ELC_EVENT05_LINK_ADCEVENT1, ELC_EVENT_INTP1_IRQn);    // ELSR05: INTP1
    ELC_Start(ELC_EVENT06_LINK_ADCEVENT2, ELC_EVENT_INTP2_IRQn);    // ELSR06: INTP2

    // Configure INTP0-2
    INTP_Init(0x07,INTP_RISING);    // Enable INTP0-2(PA08/PA09/PA10) rising edge
    INTP_Start(0x07);

#if 1
    // INTP input
    PORT->PSETC = 0x01;             // Connect PC00 to PA08 as INTP0 rising edge input
    delayMS(10);
    PORT->PSETC = 0x02;             // Connect PC01 to PA09 as INTP1 rising edge input
    delayMS(10);
    PORT->PSETC = 0x04;             // Connect PC02 to PA10 as INTP2 rising edge input
    delayMS(10);
#endif

    while(1){
        if (g_adc_refresh_flag.bit.req_adc2){
            printf("ANI200=%03x, vol=%2.3fV\n",adc2_buf[0],adc2_buf[0]*voltage/4096);
            printf("ANI201=%03x, vol=%2.3fV\n",adc2_buf[1],adc2_buf[1]*voltage/4096);
            g_adc_refresh_flag.bit.req_adc2 = 0;        // clear req
        }
        if (g_adc_refresh_flag.bit.req_adc2_b){
            printf("ANI206=%03x, vol=%2.3fV\n",adc2_b_buf[6],adc2_b_buf[6]*voltage/4096);
            printf("ANI207=%03x, vol=%2.3fV\n",adc2_b_buf[7],adc2_b_buf[7]*voltage/4096);
            g_adc_refresh_flag.bit.req_adc2_b = 0;        // clear req
        }
        if (g_adc_refresh_flag.bit.req_adc2_c){
            printf("ANI208=%03x, vol=%2.3fV\n",adc2_c_buf[8],adc2_c_buf[8]*voltage/4096);
            printf("ANI209=%03x, vol=%2.3fV\n",adc2_c_buf[9],adc2_c_buf[9]*voltage/4096);
            g_adc_refresh_flag.bit.req_adc2_c = 0;        // clear req
        }

        delayMS(500);
        PORT->PSETC = 0x4800;   // PC14,PC11 output "H"
        delayMS(500);
        PORT->PCLRC = 0x4800;   // PC14,PC11 output "L"
    }
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

