/***********************************************************************************************************************
* @file    aes.c
* @brief   This file implements device driver for AES module.
* @version 1.0.0 
* @author	 Chenzongfu
* @date    2023/1/30
***********************************************************************************************************************/
/*
 * Copyright (c) 2023 Cmsemicon Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "BAT32G439.h"
//#include "userdefine.h"
#include "aes.h"

/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
//----------------------------------------------------------------------------------------------------------------------


/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma clang diagnostic ignored "-Wcast-align"
#pragma clang diagnostic ignored "-Wsign-conversion"
#pragma clang diagnostic ignored "-Wimplicit-int-conversion"
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/


/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: AES_Init
* @brief  This function initializes the AES module.
* @param  pAESInit - the pointer of the AES_InitTypeDef structure
* @return None
***********************************************************************************************************************/
void AES_Init(AES_InitTypeDef* pAESInit)
{
	CGC->PERA |= CGC_PERA_AESEN_Msk;
	AES->MREG |= AES_MREG_core_en_Msk;
	
	while(!(AES->CREG&AES_CREG_c_wrrdy_Msk));
	AES->CREG = (pAESInit->encrypt_mode<<AES_CREG_chain_Pos) | (pAESInit->key_length<<AES_CREG_key_len_Pos);//| (pAESInit->run_mode<<AES_CREG_inv_cipher_Pos) 
	
	while(!(AES->CREG&AES_CREG_iv_wrrdy_Msk));
	AES->IV = pAESInit->pIVdata[0];
	AES->IV = pAESInit->pIVdata[1];
	AES->IV = pAESInit->pIVdata[2];
	AES->IV = pAESInit->pIVdata[3];
	
	while(!(AES->CREG&AES_CREG_k0_wrrdy_Msk));
	AES->KEY0 = pAESInit->pKeydata[0];
	AES->KEY0 = pAESInit->pKeydata[1];
	AES->KEY0 = pAESInit->pKeydata[2];
	AES->KEY0 = pAESInit->pKeydata[3];
	if(KEY_LENGTH_256==pAESInit->key_length)
	{
		while(!(AES->CREG&AES_CREG_k1_wrrdy_Msk));
		AES->KEY1 = pAESInit->pKeydata[4];
		AES->KEY1 = pAESInit->pKeydata[5];
		AES->KEY1 = pAESInit->pKeydata[6];
		AES->KEY1 = pAESInit->pKeydata[7];
	}
}

/***********************************************************************************************************************
* Function Name: AES_Encrypt
* @brief  This function encrypt data.
* @param  pAESInit - the pointer of the AES_InitTypeDef structure
* @param  pInputdata - the pointer of the input data
* @param  pOutputdata - the pointer of the output data
* @return None
***********************************************************************************************************************/
void AES_Encrypt(AES_InitTypeDef* pAESInit,uint8_t* pInputdata,uint8_t* pOutputdata)
{
	uint32_t* pU32dat;
	
	while(!(AES->CREG&AES_CREG_k0_wrrdy_Msk));
	AES->KEY0 = pAESInit->pKeydata[0];
	AES->KEY0 = pAESInit->pKeydata[1];
	AES->KEY0 = pAESInit->pKeydata[2];
	AES->KEY0 = pAESInit->pKeydata[3];
	if(KEY_LENGTH_256==pAESInit->key_length)
	{
		while(!(AES->CREG&AES_CREG_k1_wrrdy_Msk));
		AES->KEY1 = pAESInit->pKeydata[4];
		AES->KEY1 = pAESInit->pKeydata[5];
		AES->KEY1 = pAESInit->pKeydata[6];
		AES->KEY1 = pAESInit->pKeydata[7];
	}
	
	while(!(AES->CREG&AES_CREG_c_wrrdy_Msk));
	AES->CREG &= ~(AES_CREG_inv_cipher_Msk);
	AES->CREG |= (ENCRYPT_MODE<<AES_CREG_inv_cipher_Pos);
	
	if(pAESInit->encrypt_mode > AES_ECB_MODE)
	{
		while(!(AES->CREG&AES_CREG_iv_wrrdy_Msk));
		AES->IV = pAESInit->pIVdata[0];
		AES->IV = pAESInit->pIVdata[1];
		AES->IV = pAESInit->pIVdata[2];
		AES->IV = pAESInit->pIVdata[3];
	}
	
	pU32dat = (uint32_t*)pInputdata;
	while(!(AES->CREG&AES_CREG_wrrdy_Msk));
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	
	
	pU32dat = (uint32_t*)pOutputdata;
	while(!(AES->CREG&AES_CREG_rdrdy_Msk));
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	
}
/***********************************************************************************************************************
* Function Name: AES_Decrypt
* @brief  This function decrypt data.
* @param  pAESInit - the pointer of the AES_InitTypeDef structure
* @param  pInputdata - the pointer of the input data
* @param  pOutputdata - the pointer of the output data
* @return None
***********************************************************************************************************************/
void AES_Decrypt(AES_InitTypeDef* pAESInit,uint8_t* pInputdata,uint8_t* pOutputdata)
{
	uint32_t* pU32dat;
	
	while(!(AES->CREG&AES_CREG_k0_wrrdy_Msk));
	AES->KEY0 = pAESInit->pKeydata[0];
	AES->KEY0 = pAESInit->pKeydata[1];
	AES->KEY0 = pAESInit->pKeydata[2];
	AES->KEY0 = pAESInit->pKeydata[3];
	if(KEY_LENGTH_256==pAESInit->key_length)
	{
		while(!(AES->CREG&AES_CREG_k1_wrrdy_Msk));
		AES->KEY1 = pAESInit->pKeydata[4];
		AES->KEY1 = pAESInit->pKeydata[5];
		AES->KEY1 = pAESInit->pKeydata[6];
		AES->KEY1 = pAESInit->pKeydata[7];
	}
	
	while(!(AES->CREG&AES_CREG_c_wrrdy_Msk));
	AES->CREG &= ~(AES_CREG_inv_cipher_Msk);
	AES->CREG |= (DECRYPT_MODE<<AES_CREG_inv_cipher_Pos);
	
	if(pAESInit->encrypt_mode > AES_ECB_MODE)
	{
		while(!(AES->CREG&AES_CREG_iv_wrrdy_Msk));
		AES->IV = pAESInit->pIVdata[0];
		AES->IV = pAESInit->pIVdata[1];
		AES->IV = pAESInit->pIVdata[2];
		AES->IV = pAESInit->pIVdata[3];
	}
	
	pU32dat = (uint32_t*)pInputdata;
	while(!(AES->CREG&AES_CREG_wrrdy_Msk));
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	AES->DWIN = *pU32dat++;
	
	
	pU32dat = (uint32_t*)pOutputdata;
	while(!(AES->CREG&AES_CREG_rdrdy_Msk));
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	*pU32dat++ = AES->DWIN;
	
}


/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
