/***********************************************************************************************************************
* @file    aes.h
* @brief   device driver header file for AES module.
* @version 1.0.0
* @author	 Chenzongfu
* @date    2023/1/30
***********************************************************************************************************************/
/*
 * Copyright (c) 2023 Cmsemicon Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
#ifndef __AES_H__
#define __AES_H__


#define   __I     volatile const       /*!< Defines 'read only' permissions */

#define     __O     volatile             /*!< Defines 'write only' permissions */
#define     __IO    volatile             /*!< Defines 'read / write' permissions */

/* following defines should be used for structure members */
#define     __IM     volatile const      /*! Defines 'read only' structure member permissions */
#define     __OM     volatile            /*! Defines 'write only' structure member permissions */
#define     __IOM    volatile            /*! Defines 'read / write' structure member permissions */


#define ENCRYPT_MODE 0
#define DECRYPT_MODE 1

#define AES_ECB_MODE 0
#define AES_CBC_MODE 1
#define AES_CTR_MODE 2

#define KEY_LENGTH_128 0
#define KEY_LENGTH_256 1


/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/

/*AES define*/
typedef struct {
	uint8_t encrypt_mode;
	uint8_t run_mode;
	uint8_t key_length;
	uint32_t* pIVdata;
	uint32_t* pKeydata;
}AES_InitTypeDef;

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
//=================================================================================================================
void AES_Init(AES_InitTypeDef* pAESInit);
void AES_Encrypt(AES_InitTypeDef* pAESInit,uint8_t* pInputdata,uint8_t* pOutputdata);
void AES_Decrypt(AES_InitTypeDef* pAESInit,uint8_t* pInputdata,uint8_t* pOutputdata);


#endif
