/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G439
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2023/1/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "clk.h"
#include "sci.h"
#include "gpt.h"
#include "isr.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void hard_fault_handler_c(unsigned int * hardfault_args, unsigned lr_value);
uint32_t toggleflag;

//---------------------------------- delayMS -------------------------------------

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks){}
}

void GPIO_Init(void)
{
	PORT->PMB = 0x0000;
	PORT->PMCB = 0x0000;
}

//---------------------------------- ISR -------------------------------------
void GPT0_ELCCMPA0_ISR(void)
{
	GPT_ClearIntFlag(GPT00, GPT_GTST_TCFA_BIT);
	
//	toggleflag ^= 1<<7;//PB7
//	if(toggleflag&(1<<7))
//	{
//		PORT->PSETB = 1<<7;//set PB7
//	}
//	else
//	{
//		PORT->PCLRB = 1<<7;//clr PB7
//	}
}

void GPT0_ELCOVF0_ISR(void)
{
	GPT_ClearIntFlag(GPT00, GPT_GTST_TCFPO_BIT);
	
	toggleflag ^= 1<<7;//PB7
	if(toggleflag&(1<<7))
	{
		PORT->PSETB = 1<<7;//set PB7
	}
	else
	{
		PORT->PCLRB = 1<<7;//clr PB7
	}
}

void GPT0_ELCCMPA1_ISR(void)
{
	GPT_ClearIntFlag(GPT01, GPT_GTST_TCFA_BIT);
	
	toggleflag ^= 1<<6;//PB6
	if(toggleflag&(1<<6))
	{
		PORT->PSETB = 1<<6;//set PB6
	}
	else
	{
		PORT->PCLRB = 1<<6;//clr PB6
	}
}

void GPT0_ELCCMPA2_ISR(void)
{
	GPT_ClearIntFlag(GPT02, GPT_GTST_TCFA_BIT);
	
	toggleflag ^= 1<<15;//PB15
	if(toggleflag&(1<<15))
	{
		PORT->PSETB = 1<<15;//set PB15
	}
	else
	{
		PORT->PCLRB = 1<<15;//clr PB15
	}
}

//---------------------------------- GPT_Init -------------------------------------
void GPT_Init(void)
{
	
	GPT_InitTypeDef GPTInitStruct={0};

	GPTInitStruct.clockprescaler = TIMER_SOURCE_DIV_1;
	GPTInitStruct.timermode = TIMER_MODE_TRIANGLE_WAVE_SYMMETRIC_PWM;//TIMER_MODE_PWM;
	GPTInitStruct.direction = TIMER_DIRECTION_UP;
	GPTInitStruct.period = 64000;
	GPTInitStruct.dutyA = 64000/4;
	GPTInitStruct.dutyB = 64000/4;
	GPTInitStruct.deadtime = 640;
	GPTInitStruct.channelA_output_func = OUTPUT_LOW_INIT_RETAIN_END_TOGGLE_COMP;
	GPTInitStruct.channelB_output_func = OUTPUT_HIGH_INIT_RETAIN_END_TOGGLE_COMP;
	GPTInitStruct.channelA_output_enable = FUNC_ENABLE;
	GPTInitStruct.channelB_output_enable = FUNC_ENABLE;
	GPTInitStruct.period_buffer = TIMER_SINGLE_BUFFER;
	GPTInitStruct.channelA_buffer = TIMER_SINGLE_BUFFER;
	GPTInitStruct.channelB_buffer = TIMER_SINGLE_BUFFER;
	
	GPT0_Open(GPT00, &GPTInitStruct);
	GPT0_Open(GPT01, &GPTInitStruct);
	GPT0_Open(GPT02, &GPTInitStruct);
	GPT0_Open(GPT03, &GPTInitStruct);
	
	GPT1_Open(GPT10, &GPTInitStruct);
	GPT1_Open(GPT11, &GPTInitStruct);
	GPT1_Open(GPT12, &GPTInitStruct);
	GPT1_Open(GPT13, &GPTInitStruct);
	GPT1_Open(GPT14, &GPTInitStruct);
	GPT1_Open(GPT15, &GPTInitStruct);
	GPT1_Open(GPT16, &GPTInitStruct);
	GPT1_Open(GPT17, &GPTInitStruct);
	
	GTIOC00A_PORT_SETTING();
	GTIOC00B_PORT_SETTING();
	GTIOC01A_PORT_SETTING();
	GTIOC01B_PORT_SETTING();
	GTIOC02A_PORT_SETTING();
	GTIOC02B_PORT_SETTING();
	GTIOC03A_PORT_SETTING();
	GTIOC03B_PORT_SETTING();
	
	GTIOC10A_PORT_SETTING();
	GTIOC10B_PORT_SETTING();
	GTIOC11A_PORT_SETTING();
	GTIOC11B_PORT_SETTING();
	GTIOC12A_PORT_SETTING();
	GTIOC12B_PORT_SETTING();
	GTIOC13A_PORT_SETTING();
	GTIOC13B_PORT_SETTING();
	GTIOC14A_PORT_SETTING();
	GTIOC14B_PORT_SETTING();
	GTIOC15A_PORT_SETTING();
	GTIOC15B_PORT_SETTING();
	GTIOC16A_PORT_SETTING();
	GTIOC16B_PORT_SETTING();
	GTIOC17A_PORT_SETTING();
	GTIOC17B_PORT_SETTING();
	
	GPT_EnableInt(GPT00,GPT_GTINTAD_INTAEN_BIT);
	ISR_Register(GPT0_ELCCMPA0_IRQn, GPT0_ELCCMPA0_ISR);

	GPT_EnableInt(GPT00,GPT_GTINTAD_INTPOEN_BIT);
	ISR_Register(GPT0_ELCOVF0_IRQn, GPT0_ELCOVF0_ISR);
	
	GPT_EnableInt(GPT01,GPT_GTINTAD_INTAEN_BIT);
	ISR_Register(GPT0_ELCCMPA1_IRQn, GPT0_ELCCMPA1_ISR);

	GPT_EnableInt(GPT02,GPT_GTINTAD_INTAEN_BIT);
	ISR_Register(GPT0_ELCCMPA2_IRQn, GPT0_ELCCMPA2_ISR);
	
	GPT0_Start(GPT_CHANNEL00|GPT_CHANNEL01|GPT_CHANNEL02|GPT_CHANNEL03 | GPT_CHANNEL10|GPT_CHANNEL11|GPT_CHANNEL12|GPT_CHANNEL13|GPT_CHANNEL14|GPT_CHANNEL15|GPT_CHANNEL16|GPT_CHANNEL17);
//	GPT1_Start(GPT_CHANNEL10|GPT_CHANNEL11|GPT_CHANNEL12|GPT_CHANNEL13|GPT_CHANNEL14|GPT_CHANNEL15|GPT_CHANNEL16|GPT_CHANNEL17);

//	GPT0_Stop(GPT_CHANNEL00|GPT_CHANNEL01|GPT_CHANNEL02|GPT_CHANNEL03);
	
}


//-------------------------------------main----------------------------------

int main()
{
	MD_STATUS status;
	
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 0
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);
	}
	
	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// MainOSC and SubOSC enable
//-----------------------------------------------------------------------
	//CLK_Osc_Setting(OSC_EXCLK, OSC_EXCLK);
//	CLK_MainOsc_Setting(OSC_OSCILLATOR, MOSC_DIV_1);
//	CLK_SubOsc_Setting(OSC_OSCILLATOR, OSC_LOW_POWER);
	
//-----------------------------------------------------------------------
// Select PLL as system clock
//-----------------------------------------------------------------------
//  CLK_PLL_Setting(PLL_SR_fMX,PLL_DIV_1,PLL_MUL_16);
//	CLK_PLL_Start();
//	CLK_Fclk_Select(MAINCLK_FPLL);
//	while((CGC->MCKC & CGC_MCKC_CKSTR_Msk) == 0);
  SystemCoreClock = 64000000;        // PLL clock = 128MHz
	
	SysTick_Config(SystemCoreClock/1000);//1ms
	
	
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	printf("GPT PWM Test!\n");
	
	GPIO_Init();
	GPT_Init();
	
  while(1)
	{
		delayMS(250);
		
	}

}
/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	NVIC_ClearPendingIRQ(SysTick_IRQn);
	g_ticks--;
}



