/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G139
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);
static uint8_t tx_buf[3];
static uint8_t rx_buf[3];


int main()
 {
	MD_STATUS status;
	uint32_t err;
	uint32_t i;

//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);	// The baud rate cannot get at the current system clock frequency.
	}
	
	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// IIC20 transmission/reception
//-----------------------------------------------------------------------
	printf("\n");	
	printf("Please connect No.32 pin(PD02/SCL20) to SCL on partner board\n");
	printf("Please connect No.34 pin(PD00/SDA20) to SDA on partner board\n");
	printf("Press any key to continue\n");
	getchar();
	printf("\n");	
	
	IIC_Init(SCI2_IIC20);
	tx_buf[0] = 0xAA;
	tx_buf[1] = 0x55;
	tx_buf[2] = 0x5A;
	
	g_iic20_tx_end = 0;
	IIC_MasterSend(SCI2_IIC20, SLVADDR, tx_buf, 3);
	while(g_iic20_tx_end == 0);

	g_iic20_rx_end = 0;
	IIC_MasterReceive(SCI2_IIC20, SLVADDR, rx_buf, 3);
	while(g_iic20_rx_end == 0);
	
	/***********************************************************************
	*  Compare the reveived value with sent value
	*  tx_buf[0] == rx_buf[0]
	*  tx_buf[1] == rx_buf[1]
	*  tx_buf[2] == rx_buf[2]
	***********************************************************************/
	err = 0;
	for(i=0; i<sizeof(tx_buf); i++)
	{
		if(tx_buf[i] != rx_buf[i])
		{
			err = 1;
		}
	}
	if(err)
	{
		printf("IIC20 Send and Receive NG\n");
	}
	else 
	{
		printf("IIC20 Send and Receive OK\n");
	}

//-----------------------------------------------------------------------
// IIC21 transmission/reception
//-----------------------------------------------------------------------
	printf("\n");	
	printf("Please connect No.98 pin(PB15/SCL21) to SCL on partner board\n");
	printf("Please connect No.97 pin(PB14/SDA21) to SDA on partner board\n");
	printf("Press any key to continue\n");
	getchar();
	printf("\n");	
	
	IIC_Init(SCI2_IIC21);
	tx_buf[0] = 0xAA;
	tx_buf[1] = 0x55;
	tx_buf[2] = 0x5A;
	
	g_iic21_tx_end = 0;
	IIC_MasterSend(SCI2_IIC21, SLVADDR, tx_buf, 3);
	while(g_iic21_tx_end == 0);
	
	g_iic21_rx_end = 0;
	IIC_MasterReceive(SCI2_IIC21, SLVADDR, rx_buf, 3);
	while(g_iic21_rx_end == 0);
	
	/***********************************************************************
	*  Compare the reveived value with sent value
	*  tx_buf[0] == rx_buf[0]
	*  tx_buf[1] == rx_buf[1]
	*  tx_buf[2] == rx_buf[2]
	***********************************************************************/
	err = 0;
	for(i=0; i<sizeof(tx_buf); i++)
	{
		if(tx_buf[i] != rx_buf[i])
		{
			err = 1;
		}
	}
	if(err)
	{
		printf("IIC21 Send and Receive NG\n");
	}
	else
	{
		printf("IIC21 Send and Receive OK\n");
	}
    
    while(1){}

}
/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

