/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G439
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "dac.h"
#include "pga.h"
#include "adc.h"
#include "sci.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);
	

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

static uint8_t  set_value[4];
static uint16_t get_value[16];

static const double pga_gain_value[16] = {2.000,2.500,2.667,2.857,3.077,3.333,3.636,4.000,4.444,5.000,5.714,6.667,8.000,10.000,16.000,1.000};
static const uint8_t  pga_gain_symbol[16] = {PGA_GAIN_2P000,PGA_GAIN_2P500,PGA_GAIN_2P667,PGA_GAIN_2P857,PGA_GAIN_3P077,PGA_GAIN_3P636,PGA_GAIN_3P333,
											 PGA_GAIN_4P000,PGA_GAIN_4P444,PGA_GAIN_5P000,PGA_GAIN_5P714,PGA_GAIN_6P667,PGA_GAIN_8P000,PGA_GAIN_10P000,PGA_GAIN_16P000,PGA_GAIN_1P000};
int main(void)
{
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i, j;
	
//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 
	
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);
	}
	
	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// DAC channel 0 output(PB02) --> PGA00(PA00) --> ADC0 channel 0 input
//-----------------------------------------------------------------------
	DAC_Init(DAC0);
	DAC_Start(DAC0);
#if 0
    // Test PGA00
	adc_initial(ADC0, ADCx_CH0);	
	adc_software_start(ADC0);
		
	printf("PGA00 Gain Test...\n");
	for(i=0; i<sizeof(pga_gain_symbol)/sizeof(pga_gain_symbol[0]); i++)
	{
		PGA_Init(PGA00,pga_gain_symbol[i]);
		for(j=1; j<sizeof(set_value); j++)
		{
			set_value[j] = (j*256/pga_gain_value[i]/sizeof(set_value));
			DAC_SetValue(DAC0, set_value[j]);
			delayMS(10);
			get_value[j] = ADC0->ADDR0;
		}
		//-----------------------------------------------------------------------
		// Print result
		//-----------------------------------------------------------------------
		printf("DAC ----------------- ADC ------------------ PGA(X%d):\n", pga_gain_value[i]);
		for(j=1; j<sizeof(set_value); j++)
		{
			printf("DAC Set Value = 0x%02X, ", set_value[j]);
			printf("ADC Get Value = 0x%03X, ", get_value[j]);
			printf("PGA0 Gain = %5.3f\n", (double)get_value[j]/(set_value[j] << 4));		
		}
		printf("\n");
	}

	adc_software_stop(ADC0);	
	DAC_Stop(DAC0);
	PGA_DeInit(PGA00);
#else
//-----------------------------------------------------------------------
// DAC channel 0 output(PB02) --> PGA11(PA04) --> ADC1 channel 1 input
//-----------------------------------------------------------------------  
   // Test PGA11
	adc_initial(ADC1, ADCx_CH1);	
	adc_software_start(ADC1);
		
	printf("PGA11 Gain Test...\n");
	for(i=0; i<sizeof(pga_gain_symbol)/sizeof(pga_gain_symbol[0]); i++)
	{
		PGA_Init(PGA11,pga_gain_symbol[i]);
		for(j=1; j<sizeof(set_value); j++)
		{
			set_value[j] = (j*256/pga_gain_value[i]/sizeof(set_value));
			DAC_SetValue(DAC0, set_value[j]);
			delayMS(10);
			adc_software_start(ADC1);
			delayMS(10);
			get_value[j] = ADC1->ADDR1;
		}
		//-----------------------------------------------------------------------
		// Print result
		//-----------------------------------------------------------------------
		printf("DAC ----------------- ADC ------------------ PGA(X%3.2f):\n", pga_gain_value[i]);
		for(j=1; j<sizeof(set_value); j++)
		{
			printf("DAC Set Value = 0x%02X, ", set_value[j]);
			printf("ADC Get Value = 0x%03X, ", get_value[j]);
			printf("PGA11 Gain = %5.3f\n", (double)get_value[j]/(set_value[j] << 4));		
		}
		printf("\n");
	}

	adc_software_stop(ADC1);	
	DAC_Stop(DAC0);
	PGA_DeInit(PGA11);
#endif
	while(1){
		__NOP();
	}
}
/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

