/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G157
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/12/25
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
//#define RTL_SIMULATION
//#define USE_HANDSHAKE
#define TEST_SPI_MODE_0
#define TEST_SPI_MODE_3

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "spi.h"
#include "intp.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
void SysTick_Handler(void);
void delayMS(uint32_t n);
extern volatile uint8_t * gp_spi1_rx_address;         /* spi receive buffer address */
extern volatile uint16_t  g_spi1_rx_length;           /* spi receive data length */
extern volatile uint16_t  g_spi1_rx_count;            /* spi receive data count */
extern volatile uint8_t * gp_spi1_tx_address;         /* spi send buffer address */
extern volatile uint16_t  g_spi1_tx_length;           /* spi send data length */
extern volatile uint16_t  g_spi1_tx_count;            /* spi send data count */
extern volatile uint32_t  g_intp0Taken;  
static volatile uint32_t g_ticks;
static uint8_t mtx_buf[] = {0xA5, 0x5A, 0xCC, 0x33, 0x3C, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
static uint8_t mrx0_buf[13];
static uint8_t mrx3_buf[13];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
{
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i, err;
	
//-----------------------------------------------------------------------
//	Pause SPI transfer when program stops during debugging 
//-----------------------------------------------------------------------
	DBG->DBGSTOPCR |= DBG_DBGSTOPCR_FRZEN1_Msk;
	
//	for(i=0;i<100;i++);	
	
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#ifndef RTL_SIMULATION
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}
	
	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt);

//-----------------------------------------------------------------------
// Please connect the following pins:
// SCK:  PD09/NSS0  --- PC11/NSS1
// SCK:  PD14/SCK0  --- PC15/SCK1
// MOSI: PD12/MOSI0 --- PD11/MOSI1	
// MISO: PD15/MISO0 --- PD10/MISO1	
// INTP0:PA08/INTP0 --- PA08/INTP0
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
//	printf("Please connect PD14/SCK0 to PC15/SCK1\n");
//	printf("Please connect PD12/MOSI0 to PD11/MOSI1\n");
//	printf("Please connect PD15/MISO0 to PD10/MISO1\n");
//	printf("Press any key to continue\n");
//	getchar();
//	printf("\n");
#endif

//-----------------------------------------------------------------------
// Connect PA08/INTP0 to PA08 of Slave chip
// A pulse of INTP0 to inform the master that the slave is ready
// PC11/LED indicates the result of data received by mode 0
// PC14/LED indicates the result of data received by mode 3
//-----------------------------------------------------------------------
#ifdef USE_HANDSHAKE
	INTP_Init(1 << 0, INTP_RISING);
	INTP_Start(1 << 0);
#endif
	PORT->PMC  &= ~(9<<11);  // PC11,14 output mode
	PORT->PSETC   |=  (9<<11);  
	
//=======================================================================
// SPI MODE 0 Master transmission/reception
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
#ifdef TEST_SPI_MODE_0
	SPI1_MasterInit(SPI_MODE_0);

#ifdef USE_HANDSHAKE
	/* waiting slave ready */
	while(g_intp0Taken == 0);
	g_intp0Taken = 0;
#endif	

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI1_MasterSend(mtx_buf, sizeof(mtx_buf));
#ifdef SPIHS1_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(gp_spi1_tx_address != 0);
#endif	
	delayMS(5);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI1_MasterReceive(mrx0_buf, sizeof(mrx0_buf));
	
#ifdef SPIHS1_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(gp_spi1_rx_address != 0);
#endif	
	delayMS(2);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
	err = 0;
	for(i=0; i<sizeof(mrx0_buf); i++) {		
		if(mtx_buf[i] != mrx0_buf[i]) 
		{
			err++; 
		}	
	}
	if(err) {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<11);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PSETC = (1<<11); 	// PC01 output "H"
	}
	else {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<11);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PCLRC = (1<<11); 	// PC01 output "L"		
	}

#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(mrx0_buf); i++) {		
		if(mtx_buf[i] != mrx0_buf[i]) 
		{
			err++; 
			printf("mtx_value[%2d] = %02x, mrx0_value[%2d] = %02x\n", i, mtx_buf[i], i, mrx0_buf[i]);
		}	
	}
	if(err) {
		printf("SPI Mode 0 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 0 Send OR  Receive test OK!!\n");
	}	
	
#endif
#endif	
//=======================================================================
// SPI MODE 3 Master transmission/reception
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
#ifdef TEST_SPI_MODE_3
	SPI1_MasterInit(SPI_MODE_3);

#ifdef USE_HANDSHAKE
	/* waiting slave ready */
	while(g_intp0Taken == 0);
	g_intp0Taken = 0;
#endif
	
//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI1_MasterSend(mtx_buf, sizeof(mtx_buf));
	
#ifdef SPIHS1_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(gp_spi1_tx_address != 0);
#endif	
	delayMS(5);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI1_MasterReceive(mrx3_buf, sizeof(mrx3_buf));

#ifdef SPIHS1_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(gp_spi1_rx_address != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
	err = 0;
	for(i=0; i<sizeof(mrx3_buf); i++) {		
		if(mtx_buf[i] != mrx3_buf[i]) 
		{
			err++; 
		}	
	}
	if(err) {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<14);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PSETC = (1<<14); 	
	}
	else {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<14);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PCLRC = (1<<14); 		
	}
	
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(mrx3_buf); i++) {		
		if(mtx_buf[i] != mrx3_buf[i]) 
		{
			err++; 
			printf("mtx_value[%2d] = %02x, mrx3_value[%2d] = %02x\n", i, mtx_buf[i], i, mrx3_buf[i]);
		}	
	}
	if(err) {
		printf("SPI Mode 3 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 3 Send OR  Receive test OK!!\n");
	}
#endif
#endif
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}
