/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G157
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
//#define RTL_SIMULATION

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "spi.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
void delayMS(uint32_t n);
void SysTick_Handler(void);
static volatile uint32_t g_ticks;
static uint8_t mtx_buf[] = {0xA5, 0x5A, 0xCC, 0x33, 0x3C, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
static uint8_t stx_buf[] = {0x33, 0xCC, 0xC3, 0x5A, 0xA5, 0x08, 0x04, 0x02, 0x01, 0x80, 0x40, 0x20, 0x10};
static uint8_t mrx_buf[13];
static uint8_t srx_buf[13];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
{
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i, err;

//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#ifndef RTL_SIMULATION
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}

	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt);

//-----------------------------------------------------------------------
// Please connect the following pins:
// SCK:  PD09/NSS0  --- PC11/NSS1
// SCK:  PD14/SCK0  --- PC15/SCK1
// MOSI: PD12/MOSI0 --- PD11/MOSI1	
// MISO: PD15/MISO0 --- PD10/MISO1	
// INTP0:PA08/INTP0 --- PA08/INTP0
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
//	printf("Please connect PD14/SCK0 to PC15/SCK1\n");
//	printf("Please connect PD12/MOSI0 to PD11/MOSI1\n");
//	printf("Please connect PD15/MISO0 to PD10/MISO1\n");
//	printf("Press any key to continue\n");
//	getchar();
//	printf("\n");
#endif

//=======================================================================
// SPI0 MODE 0 Master transmission/reception
// SPI1 MODE 0 Slave reception/transmission
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
	SPI0_MasterInit(SPI_MODE_0);
	SPI1_SlaveInit(SPI_MODE_0);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI1_SlaveReceive(srx_buf, sizeof(srx_buf));
	SPI0_MasterSend(mtx_buf, sizeof(mtx_buf));	
#ifdef SPIHS0_WITH_DMA
	__WFI();
#else
	while(g_spi1_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI1_SlaveSend(stx_buf, sizeof(stx_buf));
	SPI0_MasterReceive(mrx_buf, sizeof(mrx_buf));
#ifdef SPIHS0_WITH_DMA
	__WFI();
#else
	while(g_spi0_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 0 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 0 Send OR  Receive test OK!!\n");
	}
#endif

//=======================================================================
// SPI0 MODE 3 Master transmission/reception
// SPI1 MODE 3 Slave reception/transmission
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
	SPI0_MasterInit(SPI_MODE_3);
	SPI1_SlaveInit(SPI_MODE_3);
//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI1_SlaveReceive(srx_buf, sizeof(srx_buf));
	SPI0_MasterSend(mtx_buf, sizeof(mtx_buf));
#ifdef SPIHS0_WITH_DMA
	__WFI();
#else
	while(g_spi1_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI1_SlaveSend(stx_buf, sizeof(stx_buf));
	SPI0_MasterReceive(mrx_buf, sizeof(mrx_buf));
#ifdef SPIHS0_WITH_DMA
	__WFI();
#else
	while(g_spi0_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 3 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 3 Send OR  Receive test OK!!\n");
	}
#endif

	while(1){}
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}
