/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G157
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/12/25
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
//#define RTL_SIMULATION
//#define USE_HANDSHAKE
#define TEST_SPI_MODE_0
#define TEST_SPI_MODE_3

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "spi.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
void SysTick_Handler(void);
void delayMS(uint32_t n);
extern volatile uint8_t * gp_spi0_rx_address;         /* spi receive buffer address */
extern volatile uint16_t  g_spi0_rx_length;           /* spi receive data length */
extern volatile uint16_t  g_spi0_rx_count;            /* spi receive data count */
extern volatile uint8_t * gp_spi0_tx_address;         /* spi send buffer address */
extern volatile uint16_t  g_spi0_tx_length;           /* spi send data length */
extern volatile uint16_t  g_spi0_tx_count;            /* spi send data count */
static volatile uint32_t g_ticks;
static uint8_t mtx_buf[] = {0xA5, 0x5A, 0xCC, 0x33, 0x3C, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
static uint8_t srx0_buf[13];
static uint8_t srx3_buf[13];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
{
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i, err;

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt);

//-----------------------------------------------------------------------
// Please connect the following pins:
// SCK:  PD09/NSS0  --- PC11/NSS1
// SCK:  PD14/SCK0  --- PC15/SCK1
// MOSI: PD12/MOSI0 --- PD11/MOSI1	
// MISO: PD15/MISO0 --- PD10/MISO1	
// INTP0:PA08/INTP0 --- PA08/INTP0	
//-----------------------------------------------------------------------
// Connect PA09 to PA08/INTP0 of Master chip
// Output a pulse to inform the master that the slave is ready
// PC11/LED indicates the result of data received by mode 0
// PC14/LED indicates the result of data received by mode 3
//-----------------------------------------------------------------------
#ifdef USE_HANDSHAKE
	INTP0_PORT_SETTING();
#endif
	PORT->PMC  &= ~(9<<11);  // PC11,PC14 output mode
	PORT->PSETC   |=  (9<<11);  
	
//=======================================================================
// SPI MODE 0 Master transmission/reception
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
#ifdef TEST_SPI_MODE_0
	SPI0_SlaveInit(SPI_MODE_0);
	
//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI0_SlaveReceive(srx0_buf, sizeof(srx0_buf));
	
#ifdef USE_HANDSHAKE
	/* PA09 output a pulse to flag SPI slave ready */
//	PORT->PMA  &= ~(1<<9);  // PC07 output mode
//	PORT->PMCA  &= ~(1<<9);
//	PORT->POMA  &= ~(1<<9);
//	PORT->PSETA = (1<<9);
//	for(i=0; i<10; i++);
//	PORT->PCLRA = (1<<9);
#endif

#ifdef SPIHS0_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(g_spi0_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI0_SlaveSend(srx0_buf, sizeof(srx0_buf));
	
#ifdef SPIHS0_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(g_spi0_tx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
	err = 0;
	for(i=0; i<sizeof(srx0_buf); i++) {		
		if(mtx_buf[i] != srx0_buf[i]) 
		{
			err++; 
		}	
	}
	if(err) {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<11);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PSETC = (1<<11); 	
	}
	else {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<11);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PCLRC = (1<<11); 		
	}
#endif  // end of TEST_SPI_MODE_0
//=======================================================================
// SPI MODE 3 Master transmission/reception
//   Mode 0: CPOL = 0, CPHA = 0
//   Mode 1: CPOL = 0, CPHA = 1
//   Mode 2: CPOL = 1, CPHA = 0
//   Mode 3: CPOL = 1, CPHA = 1
//=======================================================================
#ifdef TEST_SPI_MODE_3
	SPI0_SlaveInit(SPI_MODE_3);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI0_SlaveReceive(srx3_buf, sizeof(srx3_buf));
	
#ifdef USE_HANDSHAKE
#ifndef RTL_SIMULATION
	delayMS(50);
#endif
	/* PC07 output a pulse to flag SPI slave ready */
//	PORT->PSETC = (1<<7);
//	for(i=0; i<10; i++);
//	PORT->PCLRC = (1<<7);
#endif

#ifdef SPIHS0_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(g_spi0_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI0_SlaveSend(srx3_buf, sizeof(srx3_buf));
	
#ifdef SPIHS0_WITH_DMA
	SysTick->CTRL  &= ~SysTick_CTRL_TICKINT_Msk;    /* Disable SysTick IRQ */
	__WFI();
	SysTick->CTRL  |=  SysTick_CTRL_TICKINT_Msk;    /* Enable SysTick IRQ */
#else
	while(g_spi0_tx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
	err = 0;
	for(i=0; i<sizeof(srx3_buf); i++) {		
		if(mtx_buf[i] != srx3_buf[i]) 
		{
			err++; 
		}	
	}
	if(err) {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<14);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PSETC = (1<<14); 	
	}
	else {
		for(i=0; i<8; i++)
		{
			PORT->PC ^= (1<<14);
	#ifndef RTL_SIMULATION
			delayMS(250);
	#endif
		}
		PORT->PCLRC = (1<<14); 		
	}

#endif	// end of TEST_SPI_MODE_3
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}



