/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G179
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
//#define RTL_SIMULATION

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern void SysTick_Handler(void);
void delayMS(uint32_t n);

static volatile uint32_t g_ticks;
static uint8_t mtx_buf[] = {0xA5, 0x5A, 0xCC, 0x33, 0x3C, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
static uint8_t stx_buf[] = {0x33, 0xCC, 0xC3, 0x5A, 0xA5, 0x08, 0x04, 0x02, 0x01, 0x80, 0x40, 0x20, 0x10};
static uint8_t mrx_buf[13];
static uint8_t srx_buf[13];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
{
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i, err;

//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#ifndef RTL_SIMULATION
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);       // SCI0: UART0
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}

	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt);

//-----------------------------------------------------------------------
// Please connect the following pins:
// SCK:  No.27 pin(PD05/SCLK11) --> No.26 pin(PD13/SCLK10)
// MOSI: No.30 pin(PD03/SDO11)  --> No.17 pin(PE12/SDI10)
// MISO: No.28 pin(PD04/SDI11)  <-- No.18 pin(PE00/SDO10)
// SS: 	 No.32 pin(PD02/SS11)   --> No.16 pin(PE13/SS10)
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	printf("Please connect No.16 pin(PE13/SS10) to No.32 pin(PD02/SS11)\n");
	printf("Please connect No.26 pin(PD13/SCLKI10) to No.27 pin(PD05/SCLKO11)\n");
	printf("Please connect No.18 pin(PE00/SDO10) to No.28 pin(PD04/SDI11)\n");
	printf("Please connect No.17 pin(PE12/SDI10) to No.30 pin(PD03/SDO11)\n");
	printf("Press any key to continue\n");
	getchar();
#endif
	printf("\n");

	/* Pause SPI transfer when program stops during debugging */
	DBG->DBGSTOPCR |= DBG_DBGSTOPCR_FRZEN1_Msk;

//=======================================================================
// SPI11 MODE 0 Master transmission/reception
// SPI10 MODE 0 Slave reception/transmission
//    SPI_MODE_0 = 0x3000U,  // Mode 0: CPOL = 0, CPHA = 0; i.e. DAP = 1, CKP = 1
//    SPI_MODE_1 = 0x1000U,  // Mode 1: CPOL = 0, CPHA = 1; i.e. DAP = 0, CKP = 1
//    SPI_MODE_2 = 0x2000U,  // Mode 2: CPOL = 1, CPHA = 0; i.e. DAP = 1, CKP = 0
//    SPI_MODE_3 = 0x0000U,  // Mode 3: CPOL = 1, CPHA = 1; i.e. DAP = 0, CKP = 0
//=======================================================================
	SPI_MasterInit(SCI1_SPI11, SPI_MODE_0);
	SPI_SlaveInit(SCI1_SPI10, SPI_MODE_0);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI_SlaveReceive(SCI1_SPI10, srx_buf, sizeof(srx_buf));
	SPI_MasterSend(SCI1_SPI11, mtx_buf, sizeof(mtx_buf));
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI_SlaveSend(SCI1_SPI10, stx_buf, sizeof(stx_buf));
	SPI_MasterReceive(SCI1_SPI11, mrx_buf, sizeof(mrx_buf));
#ifdef SPI11_WITH_DMA
	__WFI();
#else
	while(g_spi11_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 0 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 0 Send OR  Receive test OK!!\n");
	}
#endif
//-----------------------------------------------------------------------
// Master Send/Receive and Slave Send/Receive
//-----------------------------------------------------------------------
	SPI_SlaveSendReceive(SCI1_SPI10, stx_buf, sizeof(stx_buf), srx_buf);
	SPI_MasterSendReceive(SCI1_SPI11, mtx_buf, sizeof(mtx_buf), mrx_buf);
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 0 Send AND Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 0 Send AND Receive test OK!!\n");
	}
#endif
//=======================================================================
// SPI11 MODE 0 Master transmission/reception
// SPI10 MODE 0 Slave reception/transmission
//    SPI_MODE_0 = 0x3000U,  // Mode 0: CPOL = 0, CPHA = 0; i.e. DAP = 1, CKP = 1
//    SPI_MODE_1 = 0x1000U,  // Mode 1: CPOL = 0, CPHA = 1; i.e. DAP = 0, CKP = 1
//    SPI_MODE_2 = 0x2000U,  // Mode 2: CPOL = 1, CPHA = 0; i.e. DAP = 1, CKP = 0
//    SPI_MODE_3 = 0x0000U,  // Mode 3: CPOL = 1, CPHA = 1; i.e. DAP = 0, CKP = 0
//=======================================================================
	SPI_MasterInit(SCI1_SPI11, SPI_MODE_3);
	SPI_SlaveInit(SCI1_SPI10, SPI_MODE_3);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI_SlaveReceive(SCI1_SPI10, srx_buf, sizeof(srx_buf));
	SPI_MasterSend(SCI1_SPI11, mtx_buf, sizeof(mtx_buf));
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI_SlaveSend(SCI1_SPI10, stx_buf, sizeof(stx_buf));
	SPI_MasterReceive(SCI1_SPI11, mrx_buf, sizeof(mrx_buf));
#ifdef SPI11_WITH_DMA
	__WFI();
#else
	while(g_spi11_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 3 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 3 Send OR  Receive test OK!!\n");
	}
#endif
//-----------------------------------------------------------------------
// Master Send/Receive and Slave Send/Receive
//-----------------------------------------------------------------------
	SPI_SlaveSendReceive(SCI1_SPI10, stx_buf, sizeof(stx_buf), srx_buf);
	SPI_MasterSendReceive(SCI1_SPI11, mtx_buf, sizeof(mtx_buf), mrx_buf);
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 3 Send AND Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 3 Send AND Receive test OK!!\n");
	}
#endif
//=======================================================================
// SPI11 MODE 0 Master transmission/reception
// SPI10 MODE 0 Slave reception/transmission
//    SPI_MODE_0 = 0x3000U,  // Mode 0: CPOL = 0, CPHA = 0; i.e. DAP = 1, CKP = 1
//    SPI_MODE_1 = 0x1000U,  // Mode 1: CPOL = 0, CPHA = 1; i.e. DAP = 0, CKP = 1
//    SPI_MODE_2 = 0x2000U,  // Mode 2: CPOL = 1, CPHA = 0; i.e. DAP = 1, CKP = 0
//    SPI_MODE_3 = 0x0000U,  // Mode 3: CPOL = 1, CPHA = 1; i.e. DAP = 0, CKP = 0
//=======================================================================
	SPI_MasterInit(SCI1_SPI11, SPI_MODE_1);
	SPI_SlaveInit(SCI1_SPI10, SPI_MODE_1);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI_SlaveReceive(SCI1_SPI10, srx_buf, sizeof(srx_buf));
	SPI_MasterSend(SCI1_SPI11, mtx_buf, sizeof(mtx_buf));
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI_SlaveSend(SCI1_SPI10, stx_buf, sizeof(stx_buf));
	SPI_MasterReceive(SCI1_SPI11, mrx_buf, sizeof(mrx_buf));
#ifdef SPI11_WITH_DMA
	__WFI();
#else
	while(g_spi11_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 1 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 1 Send OR  Receive test OK!!\n");
	}
#endif
//-----------------------------------------------------------------------
// Master Send/Receive and Slave Send/Receive
//-----------------------------------------------------------------------
	SPI_SlaveSendReceive(SCI1_SPI10, stx_buf, sizeof(stx_buf), srx_buf);
	SPI_MasterSendReceive(SCI1_SPI11, mtx_buf, sizeof(mtx_buf), mrx_buf);
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 1 Send AND Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 1 Send AND Receive test OK!!\n");
	}
#endif
//=======================================================================
// SPI11 MODE 0 Master transmission/reception
// SPI10 MODE 0 Slave reception/transmission
//    SPI_MODE_0 = 0x3000U,  // Mode 0: CPOL = 0, CPHA = 0; i.e. DAP = 1, CKP = 1
//    SPI_MODE_1 = 0x1000U,  // Mode 1: CPOL = 0, CPHA = 1; i.e. DAP = 0, CKP = 1
//    SPI_MODE_2 = 0x2000U,  // Mode 2: CPOL = 1, CPHA = 0; i.e. DAP = 1, CKP = 0
//    SPI_MODE_3 = 0x0000U,  // Mode 3: CPOL = 1, CPHA = 1; i.e. DAP = 0, CKP = 0
//=======================================================================
	SPI_MasterInit(SCI1_SPI11, SPI_MODE_2);
	SPI_SlaveInit(SCI1_SPI10, SPI_MODE_2);

//-----------------------------------------------------------------------
// Master Send and Slave Receive
//-----------------------------------------------------------------------
	SPI_SlaveReceive(SCI1_SPI10, srx_buf, sizeof(srx_buf));
	SPI_MasterSend(SCI1_SPI11, mtx_buf, sizeof(mtx_buf));
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Master Receive and Slave Send
//-----------------------------------------------------------------------
	SPI_SlaveSend(SCI1_SPI10, stx_buf, sizeof(stx_buf));
	SPI_MasterReceive(SCI1_SPI11, mrx_buf, sizeof(mrx_buf));
#ifdef SPI11_WITH_DMA
	__WFI();
#else
	while(g_spi11_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 2 Send OR  Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 2 Send OR  Receive test OK!!\n");
	}
#endif
//-----------------------------------------------------------------------
// Master Send/Receive and Slave Send/Receive
//-----------------------------------------------------------------------
	SPI_SlaveSendReceive(SCI1_SPI10, stx_buf, sizeof(stx_buf), srx_buf);
	SPI_MasterSendReceive(SCI1_SPI11, mtx_buf, sizeof(mtx_buf), mrx_buf);
#ifdef SPI10_WITH_DMA
	__WFI();
#else
	while(g_spi10_rx_count != 0);
#endif	
	delayMS(1);

//-----------------------------------------------------------------------
// Check Data
//-----------------------------------------------------------------------
#ifndef RTL_SIMULATION
	err = 0;
	for(i=0; i<sizeof(srx_buf); i++) {		
		if(mtx_buf[i] != srx_buf[i]) 
		{
			err++; 
			printf("mtx_value[%d] = %02x, srx_value[%d] = %02x\n", i, mtx_buf[i], i, srx_buf[i]);
		}	
		if(stx_buf[i] != mrx_buf[i]) 
		{
			err++; 
			printf("stx_value[%d] = %02x, mrx_value[%d] = %02x\n", i, stx_buf[i], i, mrx_buf[i]);
		}
	}
	if(err) {
		printf("SPI Mode 2 Send AND Receive test NG!!\n");
	}
	else {
		printf("SPI Mode 2 Send AND Receive test OK!!\n");
	}
#endif
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}


