/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G139
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/12/25
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "tim8.h"
#include "elc.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
 {
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i;
	
//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 
	 
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}

	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// TIM8 EventCount Test
//-----------------------------------------------------------------------
#if 1
	/*****Connect PC02 to TI10/PB00*****/	
	PORT->PMC &= ~(1<<2);
	PORT->POMC &= ~(1<<2);
	
	TM81_EventCount(TM8_CHANNEL_0, TM8_TI_POSEDGE, 10);
	
	for(i=1; i<30; i++)
	{
		PORT->PSETC =1<<2;
		delayMS(1);
		PORT->PCLRC =1<<2;
		delayMS(1);
		printf("No.%3d, TCR10 = %03d, IntTaken = %d\n", i, TM81->TCR10, NVIC_GetPendingIRQ(TM10_IRQn));
		if(NVIC_GetPendingIRQ(TM10_IRQn))
		{
			NVIC_ClearPendingIRQ(TM10_IRQn);
		}	
	}
#else
	/*****Connect PC02 to TI11/PB01*****/	
	PORT->PMC &= ~(1<<2);
	PORT->POMC &= ~(1<<2);	
	
	TM81_EventCount(TM8_CHANNEL_1, TM8_TI_POSEDGE, 10);

	for(i=1; i<30; i++)
	{
		PORT->PSETC =1<<2;
		delayMS(1);
		PORT->PCLRC =1<<2;
		delayMS(1);
		printf("No.%3d, TCR11 = %03d, IntTaken = %d\n", i, TM81->TCR11, INTC_GetPendingIRQ(TM11_IRQn));
		if(INTC_GetPendingIRQ(TM11_IRQn))
		{
			INTC_ClearPendingIRQ(TM11_IRQn);
		}		
	}
#endif
}

/***********************************************************************************************************************

* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

