/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G139
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/
/* The count value of HOCO(32MHz) clock per 1ms at 25 degrees Celsius*/
#define HOCOCNT   *((volatile uint16_t *)(0x0050064C))  
/* The frequency of HOCO(Unit:Hz) at 25 degrees Celsius */
#define HOCOFREQ  (HOCOCNT * 1000)  

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "clk.h"
#include "tim8.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
 {
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t width, period;
	 
//-----------------------------------------------------------------------
// Pause timer count when program stops during debugging
//-----------------------------------------------------------------------	 	 
	DBG->DBGSTOPCR |= DBG_DBGSTOPCR_FRZEN0_Msk;	
	 
//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 
	 
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}

	printf("Hello, UART\n");
#endif
	
//-----------------------------------------------------------------------
// Measure the high level pulse width of PB00/TI10
//-----------------------------------------------------------------------

	width = TM81_Get_PulseWidth(TM8_CHANNEL_0, TM8_PULSE_WIDTH_HIGH);
	printf("PB00/TI10 pulse width  = %d fMCK\n", width);  /* expected value: msCnt */
	
//-----------------------------------------------------------------------
// Measure the high or low level pulse width of PB00/TI10
//-----------------------------------------------------------------------

	width = TM81_Get_PulsePeriod(TM8_CHANNEL_0, TM8_PULSE_PERIOD_BOTH);
	printf("PB00/TI10 pulse width  = %d fMCK\n", width);  /* expected value: msCnt */
	
//-----------------------------------------------------------------------
// Measure the pulse period of PB00/TI10 
//-----------------------------------------------------------------------
	period = TM81_Get_PulsePeriod(TM8_CHANNEL_0,TM8_PULSE_PERIOD_RISING);
	printf("PB00/TI10 pulse period = %d fMCK\n", period);  /* expected value: msCnt * 2 */
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

