/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G139
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G439.h"
#include "userdefine.h"
#include "sci.h"
#include "tim8.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
static volatile uint32_t g_ticks;
void SysTick_Handler(void);
void delayMS(uint32_t n);

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
 {
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	 
//-----------------------------------------------------------------------
// Pause timer count when program stops during debugging
//-----------------------------------------------------------------------	 	 
	DBG->DBGSTOPCR |= DBG_DBGSTOPCR_FRZEN0_Msk;		
	 
//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	g_ticks = 1000; 	// 1000ms
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt/100); 
	 
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART_Init(SCI0, SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1); // The baud rate cannot get at the current system clock frequency.
	}

#endif

	
//-----------------------------------------------------------------------
// PB01/TO11 output square wave
//-----------------------------------------------------------------------	
	/* square wave (duty 50%)
	 *       _____       _____       _____       _____       
	 * TO11:      |_____|     |_____|     |_____|     |_____ 
	 */
	TM81_SquareOutput(TM8_CHANNEL_1, 100);
	delayMS(10);
	TM81_Channel_Stop(TM8_CHANNEL_0 | TM8_CHANNEL_1 | TM8_CHANNEL_2 | TM8_CHANNEL_3);
	
////-----------------------------------------------------------------------
//// PB01/TO11 output pwm wave
////-----------------------------------------------------------------------
//	/* PWM wave (duty 40%)
//	 *       ____        ____        ____        ____        
//	 * TO11:     |______|    |______|    |______|    |______ 
//	 */
	TM81_PWM_1Period_1Duty(100, 40);	
	delayMS(10);
	TM81_Channel_Stop(TM8_CHANNEL_0 | TM8_CHANNEL_1 | TM8_CHANNEL_2 | TM8_CHANNEL_3);

////-----------------------------------------------------------------------
//// PB01/TO11 output pwm wave
//// PD00/TO12 output pwm wave
////-----------------------------------------------------------------------
//	/* PWM wave (duty 60%, 80%)
//	 *       ______      ______      ______      ______      
//	 * TO11:       |____|      |____|      |____|      |____ 
//	 *       ________    ________    ________    ________    
//	 * TO12:         |__|        |__|        |__|        |__ 
//	 */
	TM81_PWM_1Period_2Duty(100, 60, 80);	
	delayMS(10);
	TM81_Channel_Stop(TM8_CHANNEL_0 | TM8_CHANNEL_1 | TM8_CHANNEL_2 | TM8_CHANNEL_3);

////-----------------------------------------------------------------------
//// PB01/TO11 output pwm wave
//// PD11/TO13 output pwm wave
////-----------------------------------------------------------------------
//	/* PWM wave (duty 60%, 40%)
//	 *       ______      ______      ______      ______      
//	 * TO11:       |____|      |____|      |____|      |____ 
//	 *       _________               _________                
//	 * TO13:          |_____________|         |_____________ 
//	 */
	TM81_PWM_2Period_2Duty(100, 60, 200, 80);	
	delayMS(10);
	TM81_Channel_Stop(TM8_CHANNEL_0 | TM8_CHANNEL_1 | TM8_CHANNEL_2 | TM8_CHANNEL_3);

////-----------------------------------------------------------------------
//// PB01/TO11 output pwm wave
//// PD00/TO12 output pwm wave
//// PD11/TO13 output pwm wave
////-----------------------------------------------------------------------
//	/* PWM wave (duty 20%, 50%, 90%)
//	 *       __          __          __          __           
//	 * TO11:   |________|  |________|  |________|  |________ 
//	 *       _____       _____       _____       _____       
//	 * TO12:      |_____|     |_____|     |_____|     |_____ 
//	 *       _________   _________   _________   _________   
//	 * TO13:          |_|         |_|         |_|         |_ 
//	 */
	TM81_PWM_1Period_3Duty(100, 20, 50, 90);
	delayMS(10);

    while(1){
        delayMS(100);
    }
}

/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

